import { jsx as _jsx } from "react/jsx-runtime";
import { Box, useTheme } from "@mui/joy";
import CodeMirror from "@uiw/react-codemirror";
import { markdown } from "@codemirror/lang-markdown";
import { languages } from "@codemirror/language-data";
import { syntaxTree } from "@codemirror/language";
import { TextDirectiveParser } from "./directiveParser";
import { GFM } from "@lezer/markdown";
import { RangeSetBuilder, StateField, } from "@codemirror/state";
import { EditorView, Decoration } from "@codemirror/view";
import { BulletListWidget, MarkdownTextDirectiveWidget, MarkdownWidget, TaskWidget, } from "./codeMirrorWidgets";
import { nanoid } from "@reduxjs/toolkit";
const nodeTypeToIgnore = [
    "Document",
    "TableHeader",
    "TableRow",
    "TableCell",
    "TableDelimiter",
    "Paragraph",
    "LinkMark",
    "ListItem",
    "EmphasisMark",
    "CodeMark",
    "CodeText",
    "CodeInfo",
    "LinkTitle",
    "LinkLabel",
    "Task",
];
/**
 *
 * @param {NerdMarkdownEditorProps} props Component props
 * @returns {ReactElement} NerdMarkdownEditor
 */
export function NerdMarkdownEditor(props) {
    const { onChange, value } = props;
    const theme = useTheme();
    return (_jsx(Box, { sx: {
            "& table": {
                border: "1px solid grey",
                borderRadius: "2px",
            },
            "& td": {
                border: "1px solid lightgrey",
                borderRadius: "2px",
            },
            "& p,span,div": {
                fontStyle: theme.typography.body1,
                fontFamily: theme.typography.body1.fontFamily,
            },
            "& h1": theme.typography.h1,
            "& h2": theme.typography.h2,
            "& h3": theme.typography.h3,
            "& h4": theme.typography.h4,
            "& h5": theme.typography.h5,
            "& h6": theme.typography.h6,
            "& .ATXHeading1": theme.typography.h1,
            "& .ATXHeading2": theme.typography.h2,
            "& .ATXHeading3": theme.typography.h3,
            "& .ATXHeading4": theme.typography.h4,
            "& .ATXHeading5": theme.typography.h5,
            "& .ATXHeading6": theme.typography.h6,
            "& .list": {
                marginLeft: 4,
            },
            "& .hiddenMark": {
                display: "none",
            },
            "& .blockquote": {
                ml: 4,
            },
        }, children: _jsx(CodeMirror, { value: value, onChange: (e) => onChange(e), height: "80%", basicSetup: {
                lineNumbers: false,
                foldGutter: false,
                highlightActiveLine: false,
                syntaxHighlighting: false,
            }, extensions: [
                markdown({
                    codeLanguages: languages,
                    extensions: [TextDirectiveParser, GFM],
                }),
                myPlugin,
            ] }) }));
}
const myPlugin = StateField.define({
    create(state) {
        return job(state);
    },
    update(_oldState, transaction) {
        return job(transaction.state);
    },
    provide(field) {
        return EditorView.decorations.from(field);
    },
});
/**
 *
 * @param {EditorState} state Editor state
 * @returns {RangeSet<Decoration>} List of decorations
 */
function job(state) {
    const builder = new RangeSetBuilder();
    syntaxTree(state).iterate({
        enter(node) {
            if (!nodeTypeToIgnore.includes(node.type.name)) {
                const isMultiline = state.doc.slice(node.from, node.to).lines > 1;
                const text = state.doc.sliceString(node.from, node.to);
                const cursorLine = state.doc.lineAt(state.selection.main.head);
                if (node.type.name == "TextDirective") {
                    if ((isMultiline &&
                        cursorLine.from >= node.from &&
                        cursorLine.from <= node.to) ||
                        (node.from >= cursorLine.from && node.to <= cursorLine.to))
                        return;
                    builder.add(node.from, node.to, Decoration.replace({
                        widget: new MarkdownTextDirectiveWidget(nanoid(), text),
                        inclusive: true,
                    }));
                }
                else if (node.type.name.includes("ATXHeading")) {
                    builder.add(node.from, node.to, Decoration.mark({
                        attributes: {
                            class: node.type.name,
                        },
                    }));
                }
                else if (node.type.name == "BulletList" ||
                    node.type.name == "OrderedList") {
                    builder.add(node.from, node.to, Decoration.mark({
                        tagName: "div",
                        class: "list",
                        inclusive: true,
                    }));
                }
                else if (node.type.name == "ListMark") {
                    if ((isMultiline &&
                        cursorLine.from >= node.from &&
                        cursorLine.from <= node.to) ||
                        (node.from >= cursorLine.from && node.to <= cursorLine.to))
                        return;
                    const line = state.doc.lineAt(node.from);
                    const listItem = state.doc.sliceString(line.from, line.to);
                    const match = listItem.matchAll(/(-|\+|\*) \[(x|X| )\]/gm).next();
                    if (match.value && match.value.length > 0)
                        builder.add(node.from, node.to, Decoration.mark({
                            class: "hiddenMark",
                        }));
                    else if (text == "-" || text == "+" || text == "*")
                        builder.add(node.from, node.to, Decoration.replace({
                            widget: new BulletListWidget(nanoid(), text),
                            inclusiveEnd: true,
                        }));
                }
                else if (node.type.name == "TaskMarker") {
                    if ((isMultiline &&
                        cursorLine.from >= node.from &&
                        cursorLine.from <= node.to) ||
                        (node.from >= cursorLine.from && node.to <= cursorLine.to))
                        return;
                    builder.add(node.from, node.to, Decoration.replace({
                        widget: new TaskWidget(nanoid(), text),
                        inclusive: true,
                    }));
                }
                else if (node.type.name == "Blockquote") {
                    builder.add(node.from, node.to, Decoration.mark({
                        class: "blockquote",
                    }));
                }
                else if (node.type.name == "QuoteMark") {
                    if ((isMultiline &&
                        cursorLine.from >= node.from &&
                        cursorLine.from <= node.to) ||
                        (node.from >= cursorLine.from && node.to <= cursorLine.to))
                        return;
                    builder.add(node.from, node.to, Decoration.mark({
                        class: "hiddenMark",
                    }));
                }
                else if (node.type.name == "HeaderMark") {
                    if ((isMultiline &&
                        cursorLine.from >= node.from &&
                        cursorLine.from <= node.to) ||
                        (node.from >= cursorLine.from && node.to <= cursorLine.to))
                        return;
                    builder.add(node.from, node.to + 1, Decoration.mark({
                        class: "hiddenMark",
                    }));
                }
                else if (node.type.name == "Table") {
                    if ((isMultiline &&
                        cursorLine.from >= node.from &&
                        cursorLine.from <= node.to) ||
                        (node.from >= cursorLine.from && node.to <= cursorLine.to))
                        builder.add(node.from, node.to, Decoration.mark({
                            tagName: "code",
                        }));
                    else
                        builder.add(node.from, node.to, Decoration.replace({
                            widget: new MarkdownWidget(nanoid(), text),
                            inclusive: true,
                        }));
                }
                else {
                    if ((isMultiline &&
                        cursorLine.from >= node.from &&
                        cursorLine.from <= node.to) ||
                        (node.from >= cursorLine.from && node.to <= cursorLine.to))
                        return;
                    builder.add(node.from, node.to, Decoration.replace({
                        widget: new MarkdownWidget(nanoid(), text),
                        inclusive: true,
                    }));
                }
            }
        },
    });
    return builder.finish();
}
